<?php
/*
Plugin Name: SkyPoster
Description: AI content via GPT with SEO optimization
Version: 1.0.0
Author: Nicolai Pavelev & Pavel Kudriavtsev
Plugin URI: https://skyposter.pro/
Text Domain: skyposter
License: GPLv2 or later
*/


include_once 'core.php';

$SKYPOSTER_PLUGIN_VERSION = "1.0.1";

/**
 * Plugin activation
 * @return null
 */
function skyposter_plugin_activation()
{

   // Get ID of plugin installation and license key
   $unique_id = get_option('skyposter_plugin_unique_id');
   $license_id = get_option('skyposter_plugin_license_id');

   // If this first install => create a new ID
   if ($unique_id === false) {

      // Request to backend: get new ID by domain name
      $registerResult = skyposter_register(get_site_url(), 1, $license_id);

      // If error of get a new ID
      if (!isset($registerResult['result']['hash']) || isset($registerResult['status']) && $registerResult['status'] === 'error') {

         // Deactivate the plugin
         deactivate_plugins(plugin_basename(__FILE__));
         wp_die(esc_html($registerResult['message']));

      }

      // Success: write a new installation ID to DB
      $unique_id = $registerResult['result']['hash'];
      update_option('skyposter_plugin_unique_id', $unique_id);

   }

   return null;
}

register_activation_hook(__FILE__, 'skyposter_plugin_activation');


/**
 * Load plugin
 * I18n support
 */
function skyposter_plugin_load()
{
   // TextDomain activation (i18n)
   load_plugin_textdomain('skyposter', false, dirname(plugin_basename(__FILE__)) . '/languages/');
}

add_action('plugins_loaded', 'skyposter_plugin_load');


/**
 * Add a new top-level menu with plugin settings
 */
function skyposter_plugin_add_toplevel_menu()
{
   add_menu_page(
      'SkyPoster AI Blogger',
      'SkyPoster',
      'manage_options',
      'skyposter',
      'skyposter_display_settings_page',
      plugins_url('html/img/icon_20x20.png', __FILE__),
      null // position in menu
   );
}

add_action('admin_menu', 'skyposter_plugin_add_toplevel_menu');


function skyposter_plugin_settings_link($links) {
   // Ссылка на страницу настроек
   $settings_link = '<a href="admin.php?page=skyposter&mm=Generator">'.esc_html__("Settings", "skyposter").'</a>';
   array_unshift($links, $settings_link);
   return $links;
}
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'skyposter_plugin_settings_link');


/**
 * Display the plugin settings page
 */
function skyposter_display_settings_page()
{

   global $SKYPOSTER_PLUGIN_VERSION;

   // Include the Settings page
   include('html/admin.php');

   // Load scripts

   // Frontend functions
   wp_enqueue_script(
      'skyposter_frontend_js', // unique script ID
      plugin_dir_url(__FILE__) . 'html/frontend.js', // URL
      array('wp-i18n'), // dependencies
      $SKYPOSTER_PLUGIN_VERSION,
      true // in footer of page
   );

   // Load frontend localization
   // Older versions of WP will support only English
   if (function_exists('wp_set_script_translations'))
      wp_set_script_translations('skyposter_frontend_js', 'skyposter');

   // Main
   wp_enqueue_script(
      'skyposter_core_js', // unique script ID
      plugin_dir_url(__FILE__) . 'html/mm_core.js', // URL
      array('wp-i18n'), // dependencies
      $SKYPOSTER_PLUGIN_VERSION,
      true // in footer of page
   );


}

/**
 * Load custom css for generated pages only
 */
function skyposter_enqueue_post_scripts() {
   global $post, $SKYPOSTER_PLUGIN_VERSION;
   // Check that we are on the post page (single) and the skyposter_is field exists
   if (is_singular('post') && get_post_meta($post->ID, '_skyposter_task', true)) {
      wp_enqueue_style('skyposter_css_public_post', plugin_dir_url(__FILE__) . 'css/posts.css', array(), $SKYPOSTER_PLUGIN_VERSION , 'all');
   }
}
add_action('wp_enqueue_scripts', 'skyposter_enqueue_post_scripts');


/**
 * Ajax requests to backend for writing texts
 * Made via a request on Worpress due to possible CSP
 * on different domains disallowing direct requests from the browser
 */
function skyposter_ajax_action_request()
{

   // Set time limit where it possible
   // Requests duration is about 300ms-10sec, but server may be overloaded
   set_time_limit(120);

   // Security: only for editors
   if(!current_user_can( 'edit_posts' )) {
      wp_send_json_success([
         'status' => 'error',
         'message' => esc_html__('You can`t create posts','skyposter')
      ]);
   }

   // Get request data
   $data = json_decode(file_get_contents('php://input'), true);

   // Security: verify Nonce

   // Exclude the `info' module that sends new nonces.
   if (!isset($data['data']['type'])) wp_send_json_success(['status' => 'error', 'message' => esc_html__('Bad request')]);
   if ($data['data']['type'] !== 'INFO' && wp_verify_nonce($data['nonce'], "wp_ajax_skyposter_ajax_action_request") === false) {
      //wp_send_json_success($data['data']);
      wp_send_json_success(['status' => 'error', 'message' => esc_html__('PLEASE, REFRESH THE PAGE. Bad request (incorrect nonce)')]);
   }

   // Get license and installation ID
   $unique_id = get_option('skyposter_plugin_unique_id');
   $license_id = get_option('skyposter_plugin_license_id');
   if ($data['data']['type'] === 'LICENSE_APPLY' && isset($data['data']['key']))
      $license_id = $data['data']['key'];

   // Request
   $res = skyposter_request($data['endpoint'], $data['data'], $unique_id, $license_id, wp_parse_url(get_site_url(), PHP_URL_HOST), 1);

   // If have an error, send it to browser immediately
   if ($res['status'] === 'error') {
      $res['nonce'] = wp_create_nonce('wp_ajax_skyposter_ajax_action_request');
      wp_send_json_success($res);
   }

   // SUCCESS:
   switch ($data['data']['type']) {

      // The license was successfully installed. Write it to the database
      case 'LICENSE_APPLY':
         $license_id = update_option('skyposter_plugin_license_id', $data['data']['license']);
         break;

      // Information about running tasks
      case 'INFO':

         // update nonce
         $res['nonce'] = wp_create_nonce('wp_ajax_skyposter_ajax_action_request');

         // If a task has been set
         if (isset($data['data']['taskID'])) {

            // Select only completed tasks
            if ($res['tasks'][0]['status'] !== 'COMPLETED') break;

            // The task has been completed
            if (isset($res['tasks'][0]['type']) && $res['tasks'][0]['type'] === 'ARTICLE') {

               // Post the Article

               // Get title and body of the article
               $h1_content = $res['tasks'][0]['answer']['choices'][0]['title'];
               $text_without_h1 = $res['tasks'][0]['answer']['choices'][0]['text'];

               // Post article and get ID, link and title for display it on frontend
               $post_result = skyposter_create_post($res, $res['tasks'][0]['data'], $h1_content, $text_without_h1);

               // Set info about new post into arrays
               if (is_wp_error($post_result)) {

                  // If article has not been posted
                  $res['tasks'][0]['post_name'] = $post_result->get_error_message();
                  $res['tasks'][0]['post_id'] = 0;
                  $res['tasks'][0]['post_link'] = null;
                  $res['tasks'][0]['post_date'] = null;

               } else {

                  // Success post: set info about it
                  $res['tasks'][0]['post_id'] = $post_result;
                  $res['tasks'][0]['post_link'] = get_permalink($res['tasks'][0]['post_id']);
                  $res['tasks'][0]['post_link_edit'] = get_edit_post_link($res['tasks'][0]['post_id']);
                  $res['tasks'][0]['post_name'] = $h1_content;
                  $res['tasks'][0]['post_date'] = null;

                  if(isset($res['tasks'][0]['data']['post_timestamp'])) {
                     $post = get_post($post_result);
                     $res['tasks'][0]['post_date'] = $post->post_date_gmt;
                  }

                  // Post metadata
                  skyposter_post_metadata($res, $post_result);

               }

            }

         }
         break;

      // any other requests no need additional actions
   }

   // Send data to the browser
   wp_send_json_success($res);

}

add_action('wp_ajax_skyposter_ajax_action_request', 'skyposter_ajax_action_request');

/**
 * Post the Article
 * @param $result
 * @param $inputdata
 * @param $title
 * @param $text
 * @return int|WP_Error
 */
function skyposter_create_post($res, $inputdata, $title, $text)
{

   // Clean Text
   $content = wp_kses($text, 'post');
   // $content = wp_kses('<div id="mm_article_container">'.$text.'</div>', 'post');

   // Clean Title
   $title = wp_kses($title, 'strip');

   // Post data
   $post_arr = array(
      'post_title' => $title,
      'post_content' => $content,
      'post_status' => sanitize_text_field($inputdata['post_status']),
      'post_author' => sanitize_text_field($inputdata['post_author']),
      'post_category' => array(sanitize_text_field($inputdata['post_category'])),
      'post_type' => 'post'
   );

   if(isset($inputdata['post_timestamp'])) {
      $post_arr['post_date'] = date('Y-m-d H:i:s', intval($inputdata['post_timestamp']));
      $post_arr['post_status'] = 'future';
   }

   // Post and get ID or error
   $post_id = wp_insert_post($post_arr);

   return $post_id;
}

/**
 * Create meta-tags for post
 * @param $res
 * @param $post_id
 */
function skyposter_post_metadata($res, $post_id) {

   $meta_title = $res['tasks'][0]['answer']['choices'][0]['info']['meta_title'];
   $meta_desc = $res['tasks'][0]['answer']['choices'][0]['info']['meta_description'];
   // array
   $meta_keywords = $res['tasks'][0]['answer']['choices'][0]['info']['meta_keywords'];
   // string for db
   $allKeywords = implode(', ', $meta_keywords);
   $keyword = $res['tasks'][0]['data']['keyword']['keyword'];

   //$tags = $res['tasks'][0]['answer']['choices'][0]['info']['tags'];

   // Specify that the post was created using the plugin
   update_post_meta($post_id, '_skyposter_task', sanitize_text_field($res['tasks'][0]['id']));


   // Yoast SEO
   if (defined('WPSEO_VERSION')) {
      update_post_meta($post_id, '_yoast_wpseo_title', sanitize_text_field($meta_title));
      update_post_meta($post_id, '_yoast_wpseo_metadesc', sanitize_text_field($meta_desc));
      update_post_meta($post_id, '_yoast_wpseo_focuskw', sanitize_text_field($keyword));
   }

   // All in One SEO
   if (function_exists('aioseo')) {
      update_post_meta($post_id, '_aioseop_title', sanitize_text_field($meta_title));
      update_post_meta($post_id, '_aioseop_description', sanitize_text_field($meta_desc));
      update_post_meta($post_id, '_aioseop_keywords', sanitize_text_field($allKeywords));
   }

}

?>
